"""
	Steve L. Nyemba <steve@the-phi.com>
	The Phi Technology, LLC - Store
	
	This file handles customer & plans associated with a given product/app
	We understand that a product will have multiple plans under it and we make sure that we always have a free version:
		- Having a free product insures there is no excuse not to signup users
		- If a product doesn't fall under this model we will find a way to fix it.
	- 
"""
from __future__ import division
from flask import Flask, request, session, render_template,Response
from flask_cors import CORS
import Domain
from User import User
from couchdbkit import Server, Document
import stripe
import json
from StringIO import StringIO
import re
import os
from utils.params import PARAMS
from utils.transport import Couchdb, CouchdbReader
PORT = 8100 if 'port' not in PARAMS else int(PARAMS['port']) ;
path = PARAMS['path'] #os.environ['CONFIG']
f = open(path)
CONFIG = json.loads(f.read())
CONTEXT= PARAMS['context'] if 'context' in PARAMS else ''
stripe_keys = {
    'secret_key': CONFIG['stripe']['secret'].strip(),
    'publishable_key': CONFIG['stripe']['pub'].strip()
}

stripe.api_key = stripe_keys['secret_key'].strip()


	
app = Flask(__name__)
CORS(app)
COUCHDB = Server(uri=CONFIG['couchdb']['uri']) ;
SYS_STORE = CONFIG['couchdb']
"""
	This function will set the user information to the session and update the information
	@header uid	user email address
"""
@app.route("/init/<product>",methods=['POST'])
def init(product):
	"""
		This function initializes a product to a given user, 
		if the user has provided a user identifier it will be used as her primary email. The understanding is that a product may have multiple plans under it but always a free one
		@param uid	user's email (primary)
		@param pid	plan identifier
	"""
	uid	= request.headers['uid']
	plan_id = request.headers['pid'] if 'pid' in request.headers else None
	user = User(stripe=stripe,store=CONFIG['couchdb'],product=product)
	user.subscribe(uid,plan_id)
	
	sub = None
	if 'auid' in request.headers :
		auid = request.header['auid']
		user.update(emails=auid)
		user.post()
		
		
	store = dict(CONFIG['couchdb'],**{})
	store['dbname'] = product
	store['uid'] = 'logs'
	
	sub = user.get(uid,'subscriptions')
	features = {}
	for id in sub :
		if sub[id]['active'] is True :
			features[id] = sub[id]
	user.refresh(uid)
	session['key'] = user.user_key	
	session['uid'] = uid
	return json.dumps(features),200
	
# @app.route("/init/<app_name>",methods=['POST'])
# def init(app_name):
# 	plans = []
# 	handler = None
# 	uid = request.headers['uid']
# 	plan = request.headers['pid']
# 	auid = None
# 	if 'uid' in request.headers and 'auid' not in request.headers:
# 		id = uid
# 		#couchdb = Couchdb(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid,create=False)
# 		#DB = couchdb.dbase
# 	if 'auid' in request.headers :
# 		auid = request.headers['auid']
# 		auid = json.loads(auid)if auid.startswith('[') else auid 
# 		if isinstance(auid,list) :
# 			auid = list(set(auid) - set([uid]))

# 			if len(auid) == 0:
# 				auid = None
# 				id = uid
# 			else:
# 				auid = auid[0]
# 				id = auid
# 	couchdb = Couchdb(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=id,create=False)
# 	DB = couchdb.dbase
# 		#handler.update_user(id=handler.getId(auid),uid=uid)
	
# 	handler = Domain.User(DB,stripe) ;
# 	if auid is None :
# 		handler.init(uid,plan) ;
# 	else:
# 		if couchdb.isready() == False:
# 			handler.init(auid,plan)
# 			id = handler.getId(uid) 

# 		handler.initialize(auid)
# 		id = handler.getId(auid)
# 		handler.update_user(id=id,uid=uid) ;
	
# 	plans = handler.plans()
# 	#return json.dumps(plans)
# 	_features = features(app_name)
# 	return _features
"""
	This function will update the user's email
"""
# @app.route('/init/<app_name>',methods=['PUT'])	
# def init_update(app_name):
# 	uid	= request.headers['uid']
# 	id 	= session['customer.id']
# 	couchdb = Couchdb(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=None)
# 	DB	= couchdb.dbase
# 	handler = Domain.User(DB,stripe)
# 	handler.update_user(uid=uid,id=id)
	
# 	return ('',204)
	

"""
	This function subscribes a user to a given service for an application
	This function guarantees not to duplicate subscriptions
	@resource 	name	name of the application {cloud-music}
	@header		key	service/plan
"""
@app.route('/subscribe/<product>',methods=['POST'])
def subscribe(product):
	#
	# The name is the full name of the service
	#
	resp = "0"
	user = User(stripe=stripe,store=CONFIG['couchdb'],product=product)
	if 'stripeToken' in request.form :
		stripeToken	= request.form['stripeToken']
		uid		= request.form['stripeEmail']
		tokenType	= request.form['stripeTokenType']
		amount		= request.form['amount']
		pid 		= request.form['plan']

	else:
		pid = request.headers['pid'] if 'pid' in request.headers else None
		uid = request.headers['uid']
		stripeToken = None
	user.subscribe(uid,pid,stripeToken)
	if 'auid' in request.headers :
		
		if request.headers['auid'].startswith('[') or request.headers['auid'].startswith("{") :
			auid = json.loads(request.headers['auid'])
		else:
			auid = [request.headers['auid']]
		user.update(emails=auid)
	user.refresh(uid)
	
	reader = CouchdbReader(uri=SYS_STORE['uri'],dbname=product,uid=uid,create=False)
	plans = reader.view('users/active_plan',key=user.user_key) #me['_id'])
	#session['plans'] = plans
	session['key'] = user.user_key	
	session['uid'] = uid

	return (json.dumps(plans),200)
	
def get_plans(product) :
	lproducts = stripe.Product.list()
	lproducts = [item for item in lproducts.auto_paging_iter() if item.name == product ]
	if lproducts :
		product_id = lproducts[0].id
		alias = lproducts[0].statement_descriptor
		plans = stripe.Plan.list(product=product_id)
		i = 0
		for plan in plans :
			plan['product_alias'] = alias if alias is not None else ''
			if 'features' in plan['metadata']:
				plan['metadata']['features'] = json.loads(plan['metadata']['features'])
				plans[i] = plan
			i += 1
		if plans :
			plans = plans.data
		plans.sort(key = lambda x:x.amount)	
		
		return plans
	else:
		return []

@app.route('/features/<product>')
def features(product):
	"""
		This function returns the plan/features of a user for a given application if provided a uid (email)
		if no uid are provided then the function will return all the plans/features associated with the given product
		@header uid	user's email
	"""

	plans = []
	if 'uid' in request.headers :
		uid = request.headers['uid']
		couchdb = CouchdbReader(uri=CONFIG['couchdb']['uri'],dbname=product,uid=uid,create=False)
		key  = couchdb.view("users/uid_map",key=uid)
		if key :
			key = key[0]['value']
			plans = couchdb.view('users/active_plan',key=key)
			plans =  [plan['value'] for plan in plans if 'value' in plan]
		else:
			plans = []
	else:
		plans = get_plans(product)
	#
	#  formatting plans for the output
	# 
	return json.dumps(plans)
"""
	This function returns a user's plans/status for an application
	@header uid 	user's email address
"""
@app.route('/status/<app_name>')
def status(app_name):
	
	uid = request.headers['uid']
	
	couchdb = CouchdbReader(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid,create=False)
	handler = Domain.User(couchdb.dbase,stripe)
	handler.initialize(uid)
	#lsub = handler.subscriptions()
	plans = handler.plans()
	return json.dumps(plans)
"""
	This endpoint is the signup form for a given application,
	It will allow user's to be able to signup for various plans
	@pre 'uid' in request.headers
"""	
@app.route('/signup/<product>')
def signup(product) :
	apikey = CONFIG['stripe']['pub'].strip()	

	#
	# This function returns the plans for a given application
	# We assume the application name is the prefix of the plan identifier in stripe
	#
	if 'user-info' in session :
		_info = session['user-info']
		session['uid'] = _info['uid']
	uid = session['uid'] if 'uid' in session else ''
	can_purchase = True
	if uid is None :
		uid = request.args.get('uid') if 'uid' in request.args else None
	plans = get_plans(product)
	
	#
	# @TODO: Mark the plans the current user is signed up for
	#
	platform='web' if 'platform' not in request.args else request.args['platform']
	alias = plans[0]['product_alias']
	active_plan = session['plans']  if 'plans' in session else []
	print [' ** ',uid,active_plan]
	return render_template('subscribe.html',context=CONTEXT,uid=uid,alias=alias,platform=platform,app_name=product,plans=plans,apikey=apikey)
	
@app.route('/subscribe/<name>',methods=['DELETE'])
def cancel_subscribe(name) :
	pass
"""
	This function defines if a given user is a customer or not
	We should be able to tell by how we create customers
"""
@app.route('/checkout/<app_name>',methods=['GET'])
def is_customer (app_name):
	uid = request.args.get('uid')
	pid = request.args.get('pid')
	couchdb = CouchdbReader(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid,create=False)
	r = couchdb.view('federation/uid_map',key=uid)
	id = r[0]['value']
	couchdb = CouchdbReader(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=id,create=False)
	info = couchdb.read()
	lsub = info['subscriptions']
	
	plans = [dict(dict(item['plan'],**{"status":item['status']}),**{"subscription":item['id']}) for item in lsub]
	if pid is not None:
		plans = [item for item in plans if item['id'] == pid]
	#
	# Caching the subscription identifiers so we can create an invoice later on (if need be)
	# @TODO Improve this process later on by allowing user's to pay for what they can (not everything)
	#
	session['plans'] = plans

	session['plans']
	amount = sum([item['amount'] for item in plans])	
	apikey = CONFIG['stripe']['pub'].strip()
	
	amount = amount / 100
	return render_template('bill.html',context=CONTEXT,apikey=apikey,app_name=app_name.replace('-',' '),plans=plans,total_amount=amount)

if __name__ == '__main__' :
	app.debug = True ;
	app.secret_key = '360-8y-[0v@t10n]+kr81v17y'
	app.config['MAX_CONTENT_LENGTH'] = 1600 * 1024 * 1024
	app.run(port=PORT,threaded=True,host='0.0.0.0')
